SOLVENT_DEBUG = false

solventDefaultEnabled = 'yes'
solventDefaultScale = 1.0
solventDefaultAlpha = 1.0
solventDefaultLocked = 'no'

function SolventMessage(text)
  DEFAULT_CHAT_FRAME:AddMessage("[Solvent]: " .. text, 0, 0.6, 0.6)
end

function SolventNotification(text)
  if MikSBT then
    MikSBT.DisplayMessage(text, MikSBT.DISPLAYTYPE_STATIC, false, 0, 150, 175)
  else
    SolventMessage(text)
  end
end

function TotalKeystonesFromBags()
  keystones = {
    { -- Dwarf
      ["count"] = 0,
      ["id"] = 52843,
    }, 
    { -- Draenei
      ["count"] = 0,
      ["id"] = 64394,
    }, 
    { -- Fossil
      ["count"] = 0,
      ["id"] = 0,
    }, 
    { -- Night Elf
      ["count"] = 0,
      ["id"] = 63127,
    }, 
    { -- Nerubian
      ["count"] = 0,
      ["id"] = 64396,
    }, 
    { -- Orc
      ["count"] = 0,
      ["id"] = 64392,
    }, 
    { -- Tol'vir
      ["count"] = 0,
      ["id"] = 64397,
    }, 
    { -- Troll
      ["count"] = 0,
      ["id"] = 63128,
    }, 
    { -- Vrykul
      ["count"] = 0,
      ["id"] = 64395,
    }, 
  }
 
  for bag=0,4 do
    numSlots = GetContainerNumSlots(bag)

    if SOLVENT_DEBUG then
      if SOLVENT_DEBUG then
        print(GetBagName(bag) .. ': ' .. numSlots .. ' slot bag')
      end
    end

    for slot=1,numSlots do
      itemId = GetContainerItemID(bag,slot)
      if itemId then
        texture, itemCount, locked, quality, readable, lootable, itemLink = GetContainerItemInfo(bag, slot);
        for key,hash in pairs(keystones) do
          --print(id .. " :: " .. itemId)
               
          if hash["id"] == itemId then
            itemName, otherJunk = GetItemInfo(hash["id"])
            if SOLVENT_DEBUG then
              print(" >> Found " .. itemName .. ": " .. hash["id"])
            end

            keystones[key]["count"] = itemCount
          end
        end
      end   
    end
  end

  return keystones
end

function GetCurrentArtifactInfoByRace(raceId)
  SetSelectedArtifact(raceId)
  name, otherJunk = GetSelectedArtifactInfo()
  for i=1,GetNumArtifactsByRace(raceId) do 
    iterationName, theRest = GetArtifactInfoByRace(raceId, i)
    print(name .. ' - ' .. iterationName)
    if name == iterationName then
      return iterationName, theRest
    end
  end

  return nil
end

statusBars = {}
needToTrainWarningSent = false
keystones = {}
solvableWarningSent = {}
showWhenOutOfInstance = false
function SolventGetCurrentArtifacts()
  if IsInInstance() then
    if SolventSolveFrame:IsVisible() then
      solventConfig[solventRealm][solventCharacter].enabled = 'no'
      processConfigChange()
      showWhenOutOfInstance = true
    end
    return
  elseif not SolventSolveFrame:IsVisible() and showWhenOutOfInstance then
    solventConfig[solventRealm][solventCharacter].enabled = 'yes'
    processConfigChange()
    showWhenOutOfInstance = false
  end

  SetMapToCurrentZone()
  continent = GetCurrentMapContinent();

  local raceToZoneMappings = {
    -- Kalimdor
    {
      1,
      3,
      4,
      8,
      7
    },

    -- Eastern Kingdoms
    {
      1,
      3,
      4,
      5,
      8
    },

    -- Outland
    {
      2,
      6
    },

    -- Northrend
    {
      4,
      8,
      5,
      9
    },
  }

  keystones = TotalKeystonesFromBags()

  prof1, prof2, archaeology, fishing, cooking, firstAid = GetProfessions();
  if not archaeology then 
    if not needToTrainWarningSent then
      SolventMessage("Solvent: You need to train Archaeology before I can function!")    
      needToTrainWarningSent = true
    end

    return
  end
  name, icon, skillLevel, maxSkillLevel, numAbilities, spelloffset, skillLine, skillModifier = GetProfessionInfo(archaeology)
  if SOLVENT_DEBUG then
    print(name .. ": " .. skillLevel .. "/" .. maxSkillLevel)
  end
   
  local raceCount = GetNumArchaeologyRaces();
  local lastRaceName = 'unknown'

  if SOLVENT_DEBUG then
    print("Race Count: " .. raceCount)
  end

  for i=1,raceCount do
    raceBelongsToContinent = false
    for raceKey,raceId in pairs(raceToZoneMappings[continent]) do
      if i == raceId then 
        raceBelongsToContinent = true
        break
      end
    end

    if not solvableWarningSent[i] then
      solvableWarningSent[i] = 'no'
    end

    local raceName, raceTexture, raceItemID, numFragmentsCollected, numFragmentsRequired = GetArchaeologyRaceInfo(i);
    SetSelectedArtifact(i)
    local artifactName, artifactDescription, artifactRarity, artifactIcon, hoverDescription, keystoneCount, bgTexture = GetSelectedArtifactInfo()
    --local artifactName, artifactDescription, artifactRarity, artifactIcon, hoverDescription, keystoneCount, bgTexture, firstCompletionTime, completionCount = GetArtifactInfoByRace(i, 0);
    
    --if raceBelongsToContinent then
    --  print(i .. ' - ' .. raceName)
    --end
     
    if GetNumArtifactsByRace(i) > 0 then
      if SOLVENT_DEBUG then
        DEFAULT_CHAT_FRAME:AddMessage(raceName .. ": " .. artifactName .. " - " .. numFragmentsCollected .. "/" .. numFragmentsRequired .. " - " .. keystones[i]["count"] .. " keystones in bags, " .. keystoneCount .. " keystone slots available")
      end

      if keystones[i]["count"] > keystoneCount then
        usableKeystones = keystoneCount
      else
        usableKeystones = keystones[i]["count"] 
      end
      possibleFragmentsFromKeystones = usableKeystones * 12
      solvable = false
      if (possibleFragmentsFromKeystones + numFragmentsCollected) >= numFragmentsRequired then
        solvable = true
      end

      if statusBars[i] == nil then
        if SOLVENT_DEBUG then
          DEFAULT_CHAT_FRAME:AddMessage("No Frame for " .. raceName .. ", creating...")
        end

        statusBars[i] = CreateFrame("Frame", "SolventSolveFrameRow" .. raceName, SolventSolveFrame, "SolventRowTemplate")
        statusBars[i]:SetWidth(200)
        statusBars[i]:SetHeight(12)

        statusBar = _G["SolventSolveFrameRow" .. raceName .. "StatusBar"]

        statusBar:SetStatusBarTexture("Interface\\TargetingFrame\\UI-StatusBar")
        statusBar:GetStatusBarTexture():SetHorizTile(false)
        --statusBar:SetWidth(200)
        --statusBar:SetHeight(12)
      else
        statusBar = _G["SolventSolveFrameRow" .. raceName .. "StatusBar"]
      end


      _G["SolventSolveFrameRow" .. raceName .. "Icon"]:SetFrameLevel(i+1)
      _G["SolventSolveFrameRow" .. raceName .. "Icon"]:SetAlpha(0.5)
      _G["SolventSolveFrameRow" .. raceName .. "Icon"]:RegisterForClicks('LeftButtonDown')
      _G["SolventSolveFrameRow" .. raceName .. "Icon"]:SetScript('OnClick', function(self, statusBar)
        SolventSolveByRaceId(i)
      end)

      if solvable then
        if solvableWarningSent[i] == 'no' then
          PlaySoundFile("Interface\\Addons\\Solvent\\sounds\\shooting_star.mp3")
          SolventNotification(raceName .. " Artifact '".. artifactName .. "' is now solvable!")
          solvableWarningSent[i] = 'yes'
        end
        statusBar:SetStatusBarColor(1,1,0)
        _G["SolventSolveFrameRow" .. raceName .. "Icon"]:SetAlpha(1)

      elseif artifactRarity == 0 then
        statusBar:SetStatusBarColor(0,1,0)
      else
        statusBar:SetStatusBarColor(0,0.44,0.87)
      end


      statusBar:SetMinMaxValues(0, numFragmentsRequired)
      statusBar:SetValue(numFragmentsCollected)
      statusBar.fragments:SetText(numFragmentsCollected .. ' / ' .. numFragmentsRequired)
      if strlen(artifactName) > 25 then
        statusBar.artifact:SetText(strsub(artifactName, 1, 25) .. '...')
      else
        statusBar.artifact:SetText(artifactName)
      end
      _G["SolventSolveFrameRow" .. raceName .. "Icon"].texture:SetTexture(raceTexture)
      _G["SolventSolveFrameRow" .. raceName .. "Keystones"].keystones:SetText(keystones[i]["count"] .. ' / ' .. keystoneCount)
      _G["SolventSolveFrameRow" .. raceName .. "Keystones"].keystones:SetText("+" .. possibleFragmentsFromKeystones)

    end

    if skillLevel == maxSkillLevel then
      SolventSolveFrameAddonTitleText:SetText("Solvent - Archaeology " .. skillLevel)
    else
      SolventSolveFrameAddonTitleText:SetText("Solvent - Archaeology " .. skillLevel .. ' / ' .. maxSkillLevel)
    end
    
    if _G["SolventSolveFrameRow" .. raceName] and raceBelongsToContinent then
      --print("show " .. raceName)
      _G["SolventSolveFrameRow" .. raceName]:Show()

      if lastRaceName == 'unknown' then
        statusBars[i]:SetPoint("TOP", SolventSolveFrameAddonTitleText,"BOTTOM", 0, -15)
      else
        statusBars[i]:SetPoint("TOP", _G["SolventSolveFrameRow" .. lastRaceName], "BOTTOM", 0, -15)
      end

      lastRaceName = raceName
    elseif _G["SolventSolveFrameRow" .. raceName] then
      --print("hide " .. raceName)
      statusBars[i]:Hide()
    end
  end
end

function SolventStartup()
 SolventMessage("Archaeology Addon Loaded!")
 RequestArtifactCompletionHistory()
end

surveyDetected = false
initiallyLoaded = false
function SolventHandleEvent(frame, eventName, ...)
  if eventName == 'VARIABLES_LOADED' then
    SolventProcessVariables()
    return
  end

  if SOLVENT_DEBUG then
    print(eventName)
  end

  if eventName == "ARTIFACT_HISTORY_READY" then
    initiallyLoaded = true
    SolventGetCurrentArtifacts()
  elseif initiallyLoaded and (eventName == "ARTIFACT_COMPLETE" or eventName == "BAG_UPDATE") then
    SolventGetCurrentArtifacts()
  elseif eventName == "UNIT_SPELLCAST_SUCCEEDED" then
    unit, spell, rank, notSure, spellId = ...
    if unit == "player" and spellId == 80451 then
      --print("Survey detected!")
      surveyDetected = true
    end
  elseif surveyDetected and eventName == "CURRENCY_DISPLAY_UPDATE" then
    SolventGetCurrentArtifacts()
  elseif surveyDetected and eventName == "CHAT_MSG_CURRENCY" then
    surveyDetected = false
    message, rest = ...
    if MikSBT then
      SolventNotification(message)
    end
  end
end

solventRealm = GetCVar("realmName")
solventCharacter = UnitName("player")
function SolventProcessVariables()
	if not solventConfig then 
	 	solventConfig = {}; 
	end

	if not solventConfig[solventRealm] then 
	 	solventConfig[solventRealm] = {}; 
	end

	if not solventConfig[solventRealm][solventCharacter] then 
	 	solventConfig[solventRealm][solventCharacter] = {}; 
	end
	
  if not solventConfig[solventRealm][solventCharacter].enabled then 
		solventConfig[solventRealm][solventCharacter].enabled = solventDefaultEnabled; 
	end

	if not solventConfig[solventRealm][solventCharacter].scale then 
		solventConfig[solventRealm][solventCharacter].scale = solventDefaultScale; 
	end

	if not solventConfig[solventRealm][solventCharacter].alpha then 
		solventConfig[solventRealm][solventCharacter].alpha = solventDefaultAlpha; 
	end
	
  if not solventConfig[solventRealm][solventCharacter].locked then 
		solventConfig[solventRealm][solventCharacter].locked = solventDefaultLocked; 
	end

  variablesLoaded = true;
	processConfigChange();
end

function SolventSolveByRaceId(raceId, useKeystones)
  local raceName, raceTexture, raceItemID, numFragmentsCollected, numFragmentsRequired = GetArchaeologyRaceInfo(raceId);
  SetSelectedArtifact(raceId)
  local artifactName, artifactDescription, artifactRarity, artifactIcon, hoverDescription, keystoneCount, bgTexture = GetSelectedArtifactInfo()

  if SOLVENT_DEBUG then
    print("Attempting to solve " .. raceName .. " artifact: " .. artifactName)
  end

  usableKeystones = 0
  if not keystones[raceId] then return end

  if keystones[raceId]["count"] > keystoneCount then
    usableKeystones = keystoneCount
  else
    usableKeystones = keystones[raceId]["count"] 
  end
  possibleFragmentsFromKeystones = usableKeystones * 12
  solvable = false
  if (possibleFragmentsFromKeystones + numFragmentsCollected) >= numFragmentsRequired then
    solvable = true
  end

  if solvable then
    for stoneCount=1,usableKeystones do
      if SOLVENT_DEBUG then
        print("Adding Keystone " .. stoneCount)
      end
      SocketItemToArtifact()
    end   

    if CanSolveArtifact() then
      if SOLVENT_DEBUG then
        print("Solving!")
      end
      SolveArtifact()
      solvableWarningSent[raceId] = 'no'
      return true
    else
      if SOLVENT_DEBUG then
        print("Error! Couldn't solve :(")
      end
    end
  else
    if SOLVENT_DEBUG then
      print("Could not solve! You only have " .. (possibleFragmentsFromKeystones + numFragmentsCollected) .. " fragments, need " .. numFragmentsRequired)
    end
  end 

  return false
end

function SolventSolveAllPossible()
  local raceCount = GetNumArchaeologyRaces();

  solveCount = 0
  for raceId=1,raceCount do
    if SolventSolveByRaceId(raceId) == true then
      solveCount = solveCount + 1
    end
  end

  if solveCount > 0 then 
    SolventMessage("Solved " .. solveCount .. " artifact(s).")
  else
    SolventMessage("Could not solve any artifacts.")
  end
end

function processConfigChange()
  if not variablesLoaded then return end

  if solventConfig[solventRealm][solventCharacter].enabled == 'yes' then
    SolventSolveFrame:Show()
  else
    SolventSolveFrame:Hide()
  end 

  if solventConfig[solventRealm][solventCharacter].locked == 'yes' then
    SolventSolveFrame:SetMovable(false)
  else
    SolventSolveFrame:SetMovable(true)
  end

  
  SolventSolveFrame:SetBackdropColor(1, 1, 1, solventConfig[solventRealm][solventCharacter].alpha)
  SolventSolveFrame:SetBackdropBorderColor(0.5, 0.5, 0.5, solventConfig[solventRealm][solventCharacter].alpha)
  SolventSolveFrame:SetScale(solventConfig[solventRealm][solventCharacter].scale)
end

dframe = CreateFrame("Frame", nil, UIParent)
dframe:RegisterEvent("CURRENCY_DISPLAY_UPDATE");
dframe:RegisterEvent("CHAT_MSG_CURRENCY");
dframe:RegisterEvent("ARTIFACT_HISTORY_READY")
dframe:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
dframe:RegisterEvent("ARTIFACT_COMPLETE")
dframe:RegisterEvent("VARIABLES_LOADED")
dframe:RegisterEvent("BAG_UPDATE");
dframe:SetScript("OnEvent", SolventHandleEvent);

SLASH_SOLVENT1 = '/solvent';
local function modify_display(msg, editbox)
  local command, rest = msg:match("^(%S*)%s*(.-)$");

  if command == 'hide' then
    solventConfig[solventRealm][solventCharacter].enabled = 'no'
    processConfigChange()
  elseif command == 'show' then
    if IsInInstance() then
      SolventMessage("Solvent is disabled while in an instance! Can't Show!")
      return
    end
    solventConfig[solventRealm][solventCharacter].enabled = 'yes'
    processConfigChange()
  elseif command == 'alpha' then
    alpha = tonumber(rest)
    if alpha >= 0 and alpha <= 1 then
      solventConfig[solventRealm][solventCharacter].alpha = alpha
      processConfigChange()
    end
  elseif command == 'lock' then
    solventConfig[solventRealm][solventCharacter].locked = 'yes'
    processConfigChange()
  elseif command == 'unlock' then
    solventConfig[solventRealm][solventCharacter].locked = 'no'
    processConfigChange()
  elseif command == 'scale' then
    scale = tonumber(rest)
    if scale >= 0 and scale <= 2 then
      solventConfig[solventRealm][solventCharacter].scale = scale
      processConfigChange()
    end
  else
    print("Usage: /solvent [show hide]")
  end
end
SlashCmdList["SOLVENT"] = modify_display;

SLASH_SOLVE1 = '/solve';
local function solve(command, editbox)
  command = tonumber(command)
  if command then
    SolventSolveByRaceId(command)
  else
    SolventSolveAllPossible()
  end
end
SlashCmdList["SOLVE"] = solve;
